﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;
using System.Net;
using System.Net.Sockets;

namespace HttpConsole
{
    public class CmdContext : IDisposable
    {
        public string Id { get; set; }
        public DateTime LastActivity { get; private set; }
        public IPAddress IP { get; set; }

        private Process _cmd;
        private Thread _outputThread;
        private Thread _errorThread;
        private StringBuilder _output = new StringBuilder();

        public CmdContext()
        {
            LastActivity = DateTime.Now;
            CreateID();
            CreateCmd();
        }

        private void CreateID()
        {
            var id = new StringBuilder();

            for (int i = 0; i < 8; i++)
                id.Append(Convert.ToString(Program.Random.Next(), 16));

            Id = id.ToString();
        }

        private Thread GetReaderThread(StreamReader Reader, Action<string> AppendAction)
        {
            Thread t = new Thread(x =>
            {
                while (true)
                {
                    var buffer = new char[8192];

                    var len = Reader.Read(buffer, 0, buffer.Length);

                    Array.Resize(ref buffer, len);

                    var s = new string(buffer);

                    AppendAction(s);
                }
            })
            {
                IsBackground = true
            };
            
            t.Start();

            return t;
        }

        private void CreateCmd()
        {
            _cmd = Process.Start(new ProcessStartInfo()
            {
                FileName = "cmd",
                UseShellExecute = false,
                RedirectStandardInput = true,
                RedirectStandardOutput = true,
                RedirectStandardError = true,
            });

            Action<string> outputAction = x =>
            {
                lock (_output)
                    _output.Append(x);
            };

            _outputThread = GetReaderThread(_cmd.StandardOutput, outputAction);
            _errorThread = GetReaderThread(_cmd.StandardError, outputAction);            
        }

        private void SetLastActivity()
        {
            LastActivity = DateTime.Now;
        }

        public string ReadOutput()
        {
            string o;

            lock (_output)
            {
                o = _output.ToString();

                _output = new StringBuilder();
            }

            //if (o != "")
                SetLastActivity();

            return o;
        }

        public void WriteInput(string Input)
        {
            _cmd.StandardInput.WriteLine(Input);

            SetLastActivity();
        }

        public void Dispose()
        {
            try
            {
                _outputThread.Abort();
            }
            catch (System.Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            try
            {
                _errorThread.Abort();
            }
            catch (System.Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            try
            {
                _cmd.Kill();
            }
            catch (System.Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }
    }
}
