﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace AutoSecTools.Components.Net
{
    [Serializable]
    public abstract class HttpMessage
    {
        private string _header;

        public string Header
        {
            get { return _header; }
            set 
            {
                _header = value;

                OnHeaderChangedCore();
            }
        }

        private Dictionary<string, List<string>> _headerFieldDictionary;

        public Dictionary<string, List<string>>.KeyCollection Fields
        {
            get { return _headerFieldDictionary.Keys; }
        }

        private string _body;

        public string Body
        {
            get { return _body; }
            set 
            {
                _body = value;

                OnBodyChangedCore();
            }
        }

        public List<string> this[string HeaderField]
        {
            get 
            {
                if (!_headerFieldDictionary.ContainsKey(HeaderField))
                    _headerFieldDictionary[HeaderField] = new List<string>();

                return _headerFieldDictionary[HeaderField]; 
            }
        }

        public HttpMessage()
        {
        }

        public HttpMessage(string Header, string Body)
        {
            this.Header = Header;
            this.Body = Body;            
        }

        public HttpMessage(string Message)
        {
            var bodyIndex = Message.IndexOf("\r\n\r\n");

            Header = Message.Remove(bodyIndex + 2);
            Body = Message.Substring(bodyIndex + 4);
        }

        private void OnHeaderChangedCore()
        {
            CreateHeaderFieldDictionary();

            OnHeaderChanged();
        }        

        private void OnBodyChangedCore()
        {
            OnBodyChanged();
        }

        protected virtual void OnHeaderChanged() { }

        protected virtual void OnBodyChanged() { }

        private void CreateHeaderFieldDictionary()
        {
            _headerFieldDictionary = HttpHeader.Parse(Header);            
        }

        public bool HasField(string Name)
        {
            return _headerFieldDictionary.ContainsKey(Name);
        }

        public static HttpMessage Parse(string Message)
        {
            var bodyIndex = Message.IndexOf("\r\n\r\n");

            var header = Message.Remove(bodyIndex);
            var body = Message.Substring(bodyIndex + 4);

            return Regex.IsMatch(header, @"^HTTP/1.[01]\s") ?
                new HttpResponse(header, body) as HttpMessage :
                new HttpRequest(header, body);
        }

        public override string ToString()
        {
            return Header + "\r\n" + Body;
        }
    }
}
