﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace AutoSecTools.Components.Net
{
    public class HttpRequest : HttpMessage
    {
        public string Method { get; set; }

        public Uri RequestUri { get; set; }

        public string Version { get; set; }

        private string _boundary;

        public HttpRequest() { }

        public HttpRequest(string Message)
            : base(Message) { }

        public HttpRequest(string Header, string Body)
            : base(Header, Body) { }

        protected override void OnHeaderChanged()
        {
            //var match = Regex.Match(Header, @"^([a-zA-Z]+)\s+([^\s]+)\s+(http|HTTP)/([^\s]+)");
            var match = Regex.Match(Header, @"^([a-zA-Z]+)\s+(.+)\s+(http|HTTP)/([^\s]+)");

            if (!match.Success)
                throw new InvalidOperationException("Invalid HTTP request. Error parsing first line.");

            Method = match.Groups[1].Value;
            RequestUri = new Uri(match.Groups[2].Value, UriKind.RelativeOrAbsolute);
            Version = match.Groups[4].Value;

            
            if (HasField(HttpHeader.ContentType))
            {
                var boundaryMatch = 
                    Regex.Match(this[HttpHeader.ContentType][0], @";\s*boundary=([^\s]+)");

                _boundary = boundaryMatch.Success ? boundaryMatch.Groups[1].Value : null;
            }
            
            base.OnHeaderChanged();
        }

        public override string ToString()
        {
            var header = Method + " " + RequestUri.ToString() + " HTTP/" + Version + "\r\n";

            var fields = Fields.ToList();

            Action<string, List<string>> appendHeader = (name, values) =>
            {
                foreach (var value in values)
                    header += name + ": " + value + "\r\n";
            };

            if (fields.Remove(HttpHeader.Host))
                fields.Insert(0, HttpHeader.Host);

            foreach (var field in fields)
                appendHeader(field, this[field]);

            Header = header;                

            return base.ToString();
        }
    }
}
