﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Threading;
using AutoSecTools.Components;
using AutoSecTools.Components.Net;
using AutoSecTools.Components.IO;
using AutoSecTools.Components.Collections;

namespace HttpConsole
{
    class Program
    {
        public static Random Random = new Random();

        static Dictionary<string, CmdContext> _contextTable = new Dictionary<string, CmdContext>();

        private static string _username = null, _password = null;

        private static int _port = 80;

        static void ActivityThread(object o)
        {
            while (true)
            {
                Thread.Sleep(250);

                lock (_contextTable)
                {
                    var removeKeys = new List<string>();

                    foreach (var context in _contextTable
                        .Where(x => DateTime.Now - x.Value.LastActivity > new TimeSpan(0, 1, 0)))
                    {
                        DisplayClientMessage(context.Value.IP, "timed out");

                        removeKeys.Add(context.Key);
                        context.Value.Dispose();                        
                    }

                    foreach (var k in removeKeys)
                    {
                        _contextTable.Remove(k);
                    }
                }
            }
        }

        static void ConfigServer()
        {
            Console.WriteLine("configuration");
            Console.Write("username: ");
            _username = Console.ReadLine();

            Console.Write("password: ");
            _password = Console.ReadLine();


            while (true)
            {
                Console.Write("port (default 80): ");
                var portString = Console.ReadLine();

                if (portString == "")
                    break;

                int port = 0;

                int.TryParse(portString, out port);

                if (port > 0 && port < 65536)
                {
                    _port = port;
                    break;
                }

                Console.WriteLine("invalid port");
            }
            

            Console.Clear();
        }

        static void DisplayClientMessage(IPAddress Address, string Message)
        {
            var ip = Address.ToString().PadRight(11);

            Console.WriteLine(ip + " - " + Message);
        }

        static void DisplayClientMessage(TcpClient Client, string Message)
        {
            DisplayClientMessage(((IPEndPoint)Client.Client.RemoteEndPoint).Address,
                Message);
        }

        static void AcceptTcpClientAsync(TcpClient Client)
        {
            using (var stream = Client.GetStream())
            {
                var bufferSize = 8192 * 8;
                var bytes = stream.Read(bufferSize);
                var s = bytes.GetString();

                if (s == "")
                    return;

                HttpRequest request = new HttpRequest(s);
                
                if (request["Content-Type"]
                    .Any(x => x == "application/x-www-form-urlencoded") &&
                    string.IsNullOrEmpty(request.Body))
                {
                    s += stream.ReadString(bufferSize);

                    request = new HttpRequest(s);
                }

                var c = request["Cookie"];

                string header = "HTTP/1.1 200 OK\r\n" +
                        "Content-Type: text/html; charset=utf-8\r\n" + 
                        "Connection: close\r\n";

                if (request.RequestUri.ToString() == "/favicon.ico")
                {
                    stream.WriteString(header + "\r\n");

                    return;
                }

                string id;
                CmdContext context = null;

                bool unauthenticated = false;

                //var createContext = false;

                if (c.Any())
                {
                    var cookie = new Cookies(c);

                    var f = "id";

                    if (!cookie.ContainsKey(f))
                        unauthenticated = true;
                    else
                    {
                        id = cookie[f];

                        if (!_contextTable.ContainsKey(id))
                            unauthenticated = true;
                        else
                            context = _contextTable[id];
                    }
                }
                else
                    unauthenticated = true;

                var url = request.RequestUri.ToString();

                var commandPath = "/CLI/";

                var message = "";

                if (unauthenticated)
                {
                    var reqValues = QueryString.Parse(request.Body);

                    string usernameKey = "username", passwordKey = "password";
                    
                    if (reqValues.ContainsKey(usernameKey) &&
                        reqValues.ContainsKey(passwordKey))
                    {
                        if (reqValues[usernameKey] == _username &&
                            reqValues[passwordKey] == _password)
                        {
                            DisplayClientMessage(Client, "authenticated");

                            context = new CmdContext();
                            context.IP = (Client.Client.RemoteEndPoint as IPEndPoint).Address;

                            _contextTable.Add(context.Id, context);

                            header += "Set-Cookie: id=" + context.Id + "; Path=/;\r\n";

                            unauthenticated = false;
                        }
                        else
                        {
                            DisplayClientMessage(Client, "authentication failed");

                            message = "Invalid username and/or password.";
                        }
                    }
                }

                var body = StringTemplate.ReplaceTokens(
                    Properties.Resources.login,
                    new { Message = message }
                );

                if (!unauthenticated)
                {
                    if (url == "/")
                    {
                        body = StringTemplate.ReplaceTokens(
                            Properties.Resources.index,
                            new { Jquery = Properties.Resources.jquery_min }
                        );
                    }
                    else if (url == "/Data")
                        body = context.ReadOutput();
                    else if (url.StartsWith(commandPath))
                    {
                        var command = System.Web.HttpUtility
                            .UrlDecode(url.Substring(commandPath.Length));

                        DisplayClientMessage(Client, "command: " + command);

                        context.WriteInput(command);
                    }
                }

                string response = header +
                        "\r\n" +
                        body;
                        

                stream.WriteString(response);
            }
        }

        // Todo:
        // Make exit command logout
        static void Main(string[] args)
        {
            ThreadPool.QueueUserWorkItem(ActivityThread);

            TcpListener listener;

            while (true)
            {
                ConfigServer();

                try
                {
                    listener = new TcpListener(IPAddress.Any, _port);
                    listener.Start();

                    break;
                }
                catch (System.Exception ex)
                {
                    Console.WriteLine("error starting server\r\n{0}\r\n", ex.Message);
                }                
            }

            Console.WriteLine("server started");
            
            while (true)
                try
                {
                    listener.AcceptTcpClient(AcceptTcpClientAsync);
                }
                catch (System.Exception ex)
                {
                    Console.WriteLine("error: {0}", ex.Message);
                }
                
        }
    }
}
